@@ -35,6 +35,7 @@ G_BEGIN_DECLS
 #define	CH_USB_VID				0x273f
 #define	CH_USB_PID_BOOTLOADER			0x1000
 #define	CH_USB_PID_FIRMWARE			0x1001
+#define	CH_USB_PID_FIRMWARE2			0x1004
 #define	CH_USB_PID_FIRMWARE_PLUS		0x1002
 #define	CH_USB_PID_BOOTLOADER_PLUS		0x1003
 #define	CH_USB_CONFIG				0x0001
@@ -62,7 +63,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:color_select]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -76,7 +83,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][1:color_select]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -90,7 +103,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:multiplier_value]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -104,7 +123,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][1:multiplier_value]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -118,7 +143,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][2:integral_time]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -132,7 +163,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:integral_time]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -146,7 +183,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][2:major][2:minor][2:micro]
  *
- * This command is available in bootloader and firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ✓
+ * ColorHug2      |      ✓       |      ✓
+ * ColorHug+      |      ✓       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -170,7 +213,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:index]
  * OUT: [1:retval][1:cmd][2*9:matrix_value][1:types][23:description]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -184,7 +233,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:index][4*9:matrix_value][1:types][23:description]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -198,7 +253,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:serial_number]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -212,7 +273,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][4:serial_number]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -226,7 +293,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:led_state]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -244,7 +317,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][1:led_state][1:repeat][1:on-time][1:off-time]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -258,7 +337,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][2:red][2:green][2:blue]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -274,7 +359,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:red][2:green][2:blue]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -288,7 +379,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][60:owner-name]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -302,7 +399,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][60:owner-name]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -316,7 +419,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][60:owner-email]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -330,7 +439,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][60:owner-email]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -344,7 +459,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][8:eeprom_magic]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -358,7 +479,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:count]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -375,7 +502,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -398,7 +531,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:calibration-index]
  * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -415,7 +554,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:addr]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.31
  **/
@@ -430,7 +575,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:vref]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.31
  **/
@@ -445,7 +596,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:vref]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.31
  **/
@@ -461,7 +618,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][2:red-idx][2:green-idx][2:blue-idx]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.31
  **/
@@ -475,7 +638,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:red-idx][2:green-idx][2:blue-idx]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.31
  **/
@@ -489,7 +658,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd] (but with success the device will disconnect)
  *
- * This command is available in bootloader and firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ✓
+ * ColorHug2      |      ✓       |      ✓
+ * ColorHug+      |      ✓       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -503,7 +678,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:address][1:length]
  * OUT: [1:retval][1:cmd][1:checksum][1-60:data]
  *
- * This command is only available in bootloader mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ×
+ * ColorHug2      |      ✓       |      ×
+ * ColorHug+      |      ✓       |      ×
  *
  * Since: 0.1.29
  **/
@@ -519,7 +700,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:address][2:length]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in bootloader mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ×
+ * ColorHug2      |      ✓       |      ×
+ * ColorHug+      |      ✓       |      ×
  *
  * Since: 0.1.29
  **/
@@ -534,7 +721,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:address][1:length][1:checksum][1-32:data]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in bootloader mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ×
+ * ColorHug2      |      ✓       |      ×
+ * ColorHug+      |      ✓       |      ×
  *
  * Since: 0.1.29
  **/
@@ -548,7 +741,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in bootloader mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ×
+ * ColorHug2      |      ✓       |      ×
+ * ColorHug+      |      ✓       |      ×
  *
  * Since: 0.1.29
  **/
@@ -575,8 +774,15 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][1:success]
  * OUT: [1:retval][1:cmd]
  *
- * This command is available in bootloader and firmware mode, although
- * different values of @success are permitted in each.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ✓
+ * ColorHug2      |      ✓       |      ✓
+ * ColorHug+      |      ✓       |      ✓
+
+ * Different values of @success are permitted in each mode.
  *
  * Since: 0.1.29
  **/
@@ -590,7 +796,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:scale]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -604,7 +816,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -618,7 +836,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:scale]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -632,7 +856,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -658,7 +888,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][6*2:types]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -672,7 +908,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][6*2:types]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -692,7 +934,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:hw_version]
  *
- * This command is available in bootloader and firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ✓
+ * ColorHug2      |      ✓       |      ✓
+ * ColorHug+      |      ✓       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -714,7 +962,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][30:reading_array]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -735,7 +989,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:pcb_errata]
  * OUT: [1:retval][1:cmd]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -749,7 +1009,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][2:pcb_errata]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -765,7 +1031,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][20:sha1_hash]
  * OUT: [1:retval][1:cmd]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -780,7 +1052,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][20:sha1_hash]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -796,7 +1074,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][1:measure_mode]
  * OUT: [1:retval][1:cmd]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -810,7 +1094,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:measure_mode]
  *
- * This command is available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ✓
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ×
  *
  * Since: 0.1.29
  **/
@@ -824,7 +1114,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:address][1:length]
  * OUT: [1:retval][1:cmd][1-60:data]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -838,7 +1134,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][2:address][1:length][1-60:data]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -852,7 +1154,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:temperature]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ✓
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -866,7 +1174,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][4:value]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 1.1.6
  **/
@@ -880,7 +1194,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd][4:value]
  * OUT: [1:retval][1:cmd]
  *
- * This command is only available in firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ×       |      ×
+ * ColorHug2      |      ×       |      ×
+ * ColorHug+      |      ×       |      ✓
  *
  * Since: 1.1.6
  **/
@@ -895,7 +1215,13 @@ G_BEGIN_DECLS
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd]
  *
- * This command is available in bootloader and firmware mode.
+ * This command is available under these conditions:
+ *
+ *                |  Bootloader  |  Firmware
+ * ---------------+--------------+-----------
+ * ColorHug       |      ✓       |      ✓
+ * ColorHug2      |      ✓       |      ✓
+ * ColorHug+      |      ✓       |      ✓
  *
  * Since: 0.1.29
  **/
@@ -1009,6 +1335,8 @@ typedef enum {
 	CH_ERROR_SELF_TEST_ADC_VDD,
 	CH_ERROR_SELF_TEST_ADC_VSS,
 	CH_ERROR_SELF_TEST_ADC_VREF,
+	CH_ERROR_I2C_SLAVE_ADDRESS,
+	CH_ERROR_I2C_SLAVE_CONFIG,
 	CH_ERROR_LAST
 } ChError;
 
@@ -1033,6 +1361,7 @@ typedef enum {
 	CH_DEVICE_MODE_FIRMWARE,
 	CH_DEVICE_MODE_BOOTLOADER_PLUS,
 	CH_DEVICE_MODE_FIRMWARE_PLUS,
+	CH_DEVICE_MODE_FIRMWARE2,	/* since 1.2.2 */
 	CH_DEVICE_MODE_LAST
 } ChDeviceMode;
 
