@@ -27,6 +27,7 @@
 #include <string.h>
 #include <pwd.h>
 
+#include "cd-cleanup.h"
 #include "cd-common.h"
 #include "cd-device.h"
 #include "cd-mapping-db.h"
@@ -283,9 +284,9 @@ cd_device_set_kind (CdDevice *device, CdDeviceKind kind)
 static void
 cd_device_set_object_path (CdDevice *device)
 {
-	gchar *path_tmp;
-	gchar *path_owner;
 	struct passwd *pw;
+	_cleanup_free_ gchar *path_owner = NULL;
+	_cleanup_free_ gchar *path_tmp = NULL;
 
 	/* append the uid to the object path */
 	pw = getpwuid (device->priv->owner);
@@ -301,13 +302,10 @@ cd_device_set_object_path (CdDevice *device)
 
 	/* make sure object path is sane */
 	path_owner = cd_main_ensure_dbus_path (path_tmp);
-
 	device->priv->object_path = g_build_filename (COLORD_DBUS_PATH,
 						      "devices",
 						      path_owner,
 						      NULL);
-	g_free (path_owner);
-	g_free (path_tmp);
 }
 
 /**
@@ -316,7 +314,7 @@ cd_device_set_object_path (CdDevice *device)
 void
 cd_device_set_id (CdDevice *device, const gchar *id)
 {
-	gchar *enabled_str;
+	_cleanup_free_ gchar *enabled_str = NULL;
 
 	g_return_if_fail (CD_IS_DEVICE (device));
 
@@ -337,7 +335,6 @@ cd_device_set_id (CdDevice *device, const gchar *id)
 	} else {
 		device->priv->enabled = TRUE;
 	}
-	g_free (enabled_str);
 }
 
 /**
@@ -390,6 +387,8 @@ cd_device_dbus_emit_property_changed (CdDevice *device,
 				       &builder,
 				       &invalidated_builder),
 				       NULL);
+	g_variant_builder_clear (&builder);
+	g_variant_builder_clear (&invalidated_builder);
 }
 
 /**
@@ -431,10 +430,9 @@ cd_device_dbus_emit_device_changed (CdDevice *device)
 static gboolean
 cd_device_match_qualifier (const gchar *qual1, const gchar *qual2)
 {
-	gboolean ret = FALSE;
-	gchar **split1 = NULL;
-	gchar **split2 = NULL;
 	guint i;
+	_cleanup_strv_free_ gchar **split1 = NULL;
+	_cleanup_strv_free_ gchar **split2 = NULL;
 
 	/* split into substring */
 	split1 = g_strsplit (qual1, ".", 3);
@@ -456,15 +454,11 @@ cd_device_match_qualifier (const gchar *qual1, const gchar *qual2)
 			continue;
 
 		/* failed to match substring */
-		goto out;
+		return FALSE;
 	}
 
 	/* success */
-	ret = TRUE;
-out:
-	g_strfreev (split1);
-	g_strfreev (split2);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -476,7 +470,6 @@ cd_device_find_by_qualifier (const gchar *regex,
 			     CdDeviceRelation relation)
 {
 	CdDeviceProfileItem *item;
-	CdProfile *profile = NULL;
 	const gchar *qualifier;
 	gboolean ret;
 	guint i;
@@ -491,8 +484,7 @@ cd_device_find_by_qualifier (const gchar *regex,
 		if (g_strcmp0 (regex, "*") == 0) {
 			g_debug ("anything matches, returning %s",
 				 cd_profile_get_id (item->profile));
-			profile = item->profile;
-			goto out;
+			return item->profile;
 		}
 
 		/* match with a regex */
@@ -508,13 +500,10 @@ cd_device_find_by_qualifier (const gchar *regex,
 			 ret ? "matched" : "unmatched",
 			 regex,
 			 qualifier);
-		if (ret) {
-			profile = item->profile;
-			goto out;
-		}
+		if (ret)
+			return item->profile;
 	}
-out:
-	return  profile;
+	return  NULL;
 }
 
 /**
@@ -524,7 +513,6 @@ static CdProfile *
 cd_device_find_profile_by_object_path (GPtrArray *array, const gchar *object_path)
 {
 	CdDeviceProfileItem *item;
-	CdProfile *profile = NULL;
 	gboolean ret;
 	guint i;
 
@@ -533,13 +521,10 @@ cd_device_find_profile_by_object_path (GPtrArray *array, const gchar *object_pat
 		item = g_ptr_array_index (array, i);
 		ret = (g_strcmp0 (object_path,
 				  cd_profile_get_object_path (item->profile)) == 0);
-		if (ret) {
-			profile = item->profile;
-			goto out;
-		}
+		if (ret)
+			return item->profile;
 	}
-out:
-	return  profile;
+	return NULL;
 }
 
 /**
@@ -548,12 +533,11 @@ out:
 static GVariant *
 cd_device_get_profiles_as_variant (CdDevice *device)
 {
+	CdDeviceProfileItem *item;
+	const gchar *tmp;
 	guint i;
 	guint idx = 0;
-	GVariant **profiles = NULL;
-	GVariant *value;
-	const gchar *tmp;
-	CdDeviceProfileItem *item;
+	_cleanup_free_ GVariant **profiles = NULL;
 
 	/* Object paths are assembled in this order:
 	 *
@@ -593,11 +577,9 @@ cd_device_get_profiles_as_variant (CdDevice *device)
 	}
 
 	/* format the value */
-	value = g_variant_new_array (G_VARIANT_TYPE_OBJECT_PATH,
-				     profiles,
-				     device->priv->profiles->len);
-	g_free (profiles);
-	return value;
+	return g_variant_new_array (G_VARIANT_TYPE_OBJECT_PATH,
+				    profiles,
+				    device->priv->profiles->len);
 }
 
 /**
@@ -629,7 +611,7 @@ cd_device_remove_profile (CdDevice *device,
 			     "profile object path '%s' does not exist on '%s'",
 			     profile_object_path,
 			     priv->object_path);
-		goto out;
+		return FALSE;
 	}
 
 	/* remove from the arrays */
@@ -646,8 +628,7 @@ cd_device_remove_profile (CdDevice *device,
 
 	/* emit global signal */
 	cd_device_dbus_emit_device_changed (device);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -659,7 +640,6 @@ cd_device_find_profile_relation (CdDevice *device,
 {
 	CdDevicePrivate *priv = device->priv;
 	CdDeviceProfileItem *item;
-	CdDeviceRelation relation = CD_DEVICE_RELATION_UNKNOWN;
 	guint i;
 
 	/* search profiles */
@@ -667,12 +647,10 @@ cd_device_find_profile_relation (CdDevice *device,
 		item = g_ptr_array_index (priv->profiles, i);
 		if (g_strcmp0 (profile_object_path,
 			       cd_profile_get_object_path (item->profile)) == 0) {
-			relation = item->relation;
-			goto out;
+			return item->relation;
 		}
 	}
-out:
-	return relation;
+	return CD_DEVICE_RELATION_UNKNOWN;
 }
 
 /**
@@ -718,22 +696,20 @@ cd_device_add_profile (CdDevice *device,
 {
 	CdDevicePrivate *priv = device->priv;
 	CdDeviceProfileItem *item;
-	CdProfile *profile = NULL;
 	gboolean create_item = TRUE;
-	gboolean ret = TRUE;
 	guint i;
+	_cleanup_object_unref_ CdProfile *profile = NULL;
 
 	/* is it available */
 	profile = cd_profile_array_get_by_object_path (priv->profile_array,
 						       profile_object_path);
 	if (profile == NULL) {
-		ret = FALSE;
 		g_set_error (error,
 			     CD_DEVICE_ERROR,
 			     CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 			     "profile object path '%s' does not exist",
 			     profile_object_path);
-		goto out;
+		return FALSE;
 	}
 
 	/* check it does not already exist */
@@ -753,14 +729,12 @@ cd_device_add_profile (CdDevice *device,
 				create_item = FALSE;
 				break;
 			}
-
-			ret = FALSE;
 			g_set_error (error,
 				     CD_DEVICE_ERROR,
 				     CD_DEVICE_ERROR_PROFILE_ALREADY_ADDED,
 				     "profile object path '%s' has already been added",
 				     profile_object_path);
-			goto out;
+			return FALSE;
 		}
 	}
 
@@ -789,10 +763,7 @@ cd_device_add_profile (CdDevice *device,
 
 	/* emit global signal */
 	cd_device_dbus_emit_device_changed (device);
-out:
-	if (profile != NULL)
-		g_object_unref (profile);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -804,7 +775,7 @@ cd_device_set_property_to_db (CdDevice *device,
 			      const gchar *value)
 {
 	gboolean ret;
-	GError *error = NULL;
+	_cleanup_error_free_ GError *error = NULL;
 
 	if (device->priv->object_scope != CD_OBJECT_SCOPE_DISK)
 		return;
@@ -817,7 +788,6 @@ cd_device_set_property_to_db (CdDevice *device,
 	if (!ret) {
 		g_warning ("CdDevice: failed to save property to database: %s",
 			   error->message);
-		g_error_free (error);
 	}
 }
 
@@ -828,15 +798,12 @@ static GVariant *
 cd_device_get_metadata_as_variant (CdDevice *device)
 {
 	GList *l;
-	GList *list = NULL;
 	GVariantBuilder builder;
-	GVariant *value;
+	_cleanup_list_free_ GList *list = NULL;
 
 	/* do not try to build an empty array */
-	if (g_hash_table_size (device->priv->metadata) == 0) {
-		value = g_variant_new_array (G_VARIANT_TYPE ("{ss}"), NULL, 0);
-		goto out;
-	}
+	if (g_hash_table_size (device->priv->metadata) == 0)
+		return g_variant_new_array (G_VARIANT_TYPE ("{ss}"), NULL, 0);
 
 	/* add all the keys in the dictionary to the variant builder */
 	list = g_hash_table_get_keys (device->priv->metadata);
@@ -848,10 +815,7 @@ cd_device_get_metadata_as_variant (CdDevice *device)
 				       g_hash_table_lookup (device->priv->metadata,
 							    l->data));
 	}
-	value = g_variant_builder_end (&builder);
-out:
-	g_list_free (list);
-	return value;
+	return g_variant_builder_end (&builder);
 }
 
 /**
@@ -945,26 +909,23 @@ cd_device_set_property_internal (CdDevice *device,
 				 gboolean save_in_db,
 				 GError **error)
 {
-	gboolean ret = TRUE;
 	gboolean is_metadata = FALSE;
 	CdDevicePrivate *priv = device->priv;
 
 	/* sanity check the length of the key and value */
 	if (strlen (property) > CD_DBUS_METADATA_KEY_LEN_MAX) {
-		ret = FALSE;
 		g_set_error_literal (error,
 				     CD_CLIENT_ERROR,
 				     CD_CLIENT_ERROR_INPUT_INVALID,
 				     "metadata key length invalid");
-		goto out;
+		return FALSE;
 	}
 	if (value != NULL && strlen (value) > CD_DBUS_METADATA_VALUE_LEN_MAX) {
-		ret = FALSE;
 		g_set_error_literal (error,
 				     CD_CLIENT_ERROR,
 				     CD_CLIENT_ERROR_INPUT_INVALID,
 				     "metadata value length invalid");
-		goto out;
+		return FALSE;
 	}
 
 	g_debug ("CdDevice: Attempting to set %s to %s on %s",
@@ -1016,8 +977,7 @@ cd_device_set_property_internal (CdDevice *device,
 						      property,
 						      cd_device_get_nullable_for_string (value));
 	}
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -1043,11 +1003,10 @@ cd_device_make_default (CdDevice *device,
 		        const gchar *profile_object_path,
 		        GError **error)
 {
+	CdDevicePrivate *priv = device->priv;
 	CdDeviceProfileItem *item;
 	CdProfile *profile;
 	guint i;
-	gboolean ret = FALSE;
-	CdDevicePrivate *priv = device->priv;
 
 	/* find profile */
 	profile = cd_device_find_profile_by_object_path (priv->profiles,
@@ -1058,7 +1017,7 @@ cd_device_make_default (CdDevice *device,
 			     CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 			     "profile object path '%s' does not exist for this device",
 			     profile_object_path);
-		goto out;
+		return FALSE;
 	}
 
 	/* make the profile first in the array */
@@ -1083,11 +1042,7 @@ cd_device_make_default (CdDevice *device,
 
 	/* emit global signal */
 	cd_device_dbus_emit_device_changed (device);
-
-	/* success */
-	ret = TRUE;
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -1100,13 +1055,11 @@ cd_device_set_enabled (CdDevice *device,
 {
 	CdDevicePrivate *priv = device->priv;
 	gboolean ret;
-	GError *error_local = NULL;
+	_cleanup_error_free_ GError *error_local = NULL;
 
 	/* device is already the correct state */
-	if (priv->enabled == enabled) {
-		ret = TRUE;
-		goto out;
-	}
+	if (priv->enabled == enabled)
+		return TRUE;
 
 	/* update database */
 	ret = cd_device_db_set_property (device->priv->device_db,
@@ -1119,8 +1072,7 @@ cd_device_set_enabled (CdDevice *device,
 			     CD_DEVICE_ERROR,
 			     CD_DEVICE_ERROR_INTERNAL,
 			     "%s", error_local->message);
-		g_error_free (error_local);
-		goto out;
+		return FALSE;
 	}
 
 	/* change property */
@@ -1136,8 +1088,7 @@ cd_device_set_enabled (CdDevice *device,
 
 	/* emit global signal */
 	cd_device_dbus_emit_device_changed (device);
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -1151,20 +1102,17 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 {
 	CdDevice *device = CD_DEVICE (user_data);
 	CdDevicePrivate *priv = device->priv;
+	CdDeviceRelation relation = CD_DEVICE_RELATION_UNKNOWN;
 	CdProfile *profile = NULL;
+	GVariant *tuple = NULL;
+	GVariant *value = NULL;
 	const gchar *id;
-	gboolean enabled;
-	gboolean ret;
 	const gchar *profile_object_path = NULL;
 	const gchar *property_name = NULL;
 	const gchar *property_value = NULL;
-	gchar **regexes = NULL;
-	GError *error = NULL;
+	gboolean ret;
 	guint i = 0;
-	GVariant *tuple = NULL;
-	GVariant *value = NULL;
-	gchar *strv_debug = NULL;
-	CdDeviceRelation relation = CD_DEVICE_RELATION_UNKNOWN;
+	_cleanup_error_free_ GError *error = NULL;
 
 	/* return '' */
 	if (g_strcmp0 (method_name, "AddProfile") == 0) {
@@ -1179,8 +1127,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* check the profile_object_path exists */
@@ -1203,7 +1150,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR_INTERNAL,
 							       "relation '%s' unknown, expected 'hard' or 'soft'",
 							       property_value);
-			goto out;
+			return;
 		}
 
 		/* add it */
@@ -1213,10 +1160,8 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 					     g_get_real_time (),
 					     &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_gerror (invocation,
-								error);
-			g_error_free (error);
-			goto out;
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
 		}
 
 		/* get profile id from object path */
@@ -1234,12 +1179,11 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 			if (!ret) {
 				g_warning ("CdDevice: failed to save mapping to database: %s",
 					   error->message);
-				g_error_free (error);
 			}
 		}
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	if (g_strcmp0 (method_name, "RemoveProfile") == 0) {
@@ -1254,8 +1198,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* try to remove */
@@ -1267,10 +1210,8 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 						profile_object_path,
 						&error);
 		if (!ret) {
-			g_dbus_method_invocation_return_gerror (invocation,
-								error);
-			g_error_free (error);
-			goto out;
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
 		}
 
 		/* get profile id from object path */
@@ -1288,11 +1229,10 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 		if (!ret) {
 			g_warning ("CdDevice: failed to save mapping to database: %s",
 				   error->message);
-			g_error_free (error);
 		}
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* return 's' */
@@ -1311,17 +1251,19 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR_PROFILE_DOES_NOT_EXIST,
 							       "no profile '%s' found",
 							       property_value);
-			goto out;
+			return;
 		}
 
 		tuple = g_variant_new ("(s)",
 				       cd_device_relation_to_string (relation));
 		g_dbus_method_invocation_return_value (invocation, tuple);
-		goto out;
+		return;
 	}
 
 	/* return 'o' */
 	if (g_strcmp0 (method_name, "GetProfileForQualifiers") == 0) {
+		gchar **regexes = NULL;
+		_cleanup_free_ gchar *strv_debug = NULL;
 
 		/* find the profile by the qualifier search string */
 		g_variant_get (parameters, "(^a&s)", &regexes);
@@ -1340,7 +1282,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR_PROFILING,
 							       "profiling, so ignoring '%s'",
 							       strv_debug);
-			goto out;
+			return;
 		}
 
 		/* search each regex against the profiles for this device */
@@ -1364,13 +1306,13 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR_NOTHING_MATCHED,
 							       "nothing matched expression '%s'",
 							       strv_debug);
-			goto out;
+			return;
 		}
 
 		value = g_variant_new_object_path (cd_profile_get_object_path (profile));
 		tuple = g_variant_new_tuple (&value, 1);
 		g_dbus_method_invocation_return_value (invocation, tuple);
-		goto out;
+		return;
 	}
 
 	/* return '' */
@@ -1386,8 +1328,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* check the profile_object_path exists */
@@ -1401,10 +1342,8 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 					      profile_object_path,
 					      &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_gerror (invocation,
-								error);
-			g_error_free (error);
-			goto out;
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
 		}
 
 		/* reset modification time */
@@ -1421,16 +1360,20 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 					 priv->id,
 					 id,
 					 &error);
-		if (!ret)
-			goto out;
+		if (!ret) {
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
+		}
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* return '' */
 	if (g_strcmp0 (method_name, "SetEnabled") == 0) {
 
+		gboolean enabled;
+
 		/* require auth */
 		ret = cd_main_sender_authenticated (connection,
 						    sender,
@@ -1441,8 +1384,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* set, and parse */
@@ -1452,13 +1394,11 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 			 sender, enabled ? "True" : "False");
 		ret = cd_device_set_enabled (device, enabled, &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_gerror (invocation,
-							        error);
-			g_error_free (error);
-			goto out;
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
 		}
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* return '' */
@@ -1474,8 +1414,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* set, and parse */
@@ -1490,13 +1429,11 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 						       (priv->object_scope == CD_OBJECT_SCOPE_DISK),
 						       &error);
 		if (!ret) {
-			g_dbus_method_invocation_return_gerror (invocation,
-								error);
-			g_error_free (error);
-			goto out;
+			g_dbus_method_invocation_return_gerror (invocation, error);
+			return;
 		}
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* return '' */
@@ -1512,8 +1449,7 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_AUTHENTICATE,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 
 		/* inhbit all profiles */
@@ -1527,11 +1463,10 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_INHIBIT,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* return '' */
@@ -1548,18 +1483,14 @@ cd_device_dbus_method_call (GDBusConnection *connection, const gchar *sender,
 							       CD_DEVICE_ERROR,
 							       CD_DEVICE_ERROR_FAILED_TO_UNINHIBIT,
 							       "%s", error->message);
-			g_error_free (error);
-			goto out;
+			return;
 		}
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto out;
+		return;
 	}
 
 	/* we suck */
 	g_critical ("failed to process device method %s", method_name);
-out:
-	g_free (strv_debug);
-	return;
 }
 
 /**
@@ -1592,81 +1523,45 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 {
 	CdDevice *device = CD_DEVICE (user_data);
 	CdDevicePrivate *priv = device->priv;
-	gchar **bus_names = NULL;
-	GVariant *retval = NULL;
+	_cleanup_strv_free_ gchar **bus_names = NULL;
 
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_CREATED) == 0) {
-		retval = g_variant_new_uint64 (priv->created);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODIFIED) == 0) {
-		retval = g_variant_new_uint64 (priv->modified);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODEL) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->model);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_VENDOR) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->vendor);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SERIAL) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->serial);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ENABLED) == 0) {
-		retval = g_variant_new_boolean (priv->enabled);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_COLORSPACE) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->colorspace);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_FORMAT) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->format);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODE) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->mode);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_KIND) == 0) {
-		retval = cd_device_get_nullable_for_string (cd_device_kind_to_string (priv->kind));
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ID) == 0) {
-		retval = g_variant_new_string (priv->id);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_PROFILES) == 0) {
-		retval = cd_device_get_profiles_as_variant (device);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_METADATA) == 0) {
-		retval = cd_device_get_metadata_as_variant (device);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SCOPE) == 0) {
-		retval = g_variant_new_string (cd_object_scope_to_string (priv->object_scope));
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_OWNER) == 0) {
-		retval = g_variant_new_uint32 (priv->owner);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SEAT) == 0) {
-		retval = cd_device_get_nullable_for_string (priv->seat);
-		goto out;
-	}
-	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_EMBEDDED) == 0) {
-		retval = g_variant_new_boolean (priv->embedded);
-		goto out;
-	}
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_CREATED) == 0)
+		return g_variant_new_uint64 (priv->created);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODIFIED) == 0)
+		return g_variant_new_uint64 (priv->modified);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODEL) == 0)
+		return cd_device_get_nullable_for_string (priv->model);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_VENDOR) == 0)
+		return cd_device_get_nullable_for_string (priv->vendor);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SERIAL) == 0)
+		return cd_device_get_nullable_for_string (priv->serial);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ENABLED) == 0)
+		return g_variant_new_boolean (priv->enabled);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_COLORSPACE) == 0)
+		return cd_device_get_nullable_for_string (priv->colorspace);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_FORMAT) == 0)
+		return cd_device_get_nullable_for_string (priv->format);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODE) == 0)
+		return cd_device_get_nullable_for_string (priv->mode);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_KIND) == 0)
+		return cd_device_get_nullable_for_string (cd_device_kind_to_string (priv->kind));
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ID) == 0)
+		return g_variant_new_string (priv->id);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_PROFILES) == 0)
+		return cd_device_get_profiles_as_variant (device);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_METADATA) == 0)
+		return cd_device_get_metadata_as_variant (device);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SCOPE) == 0)
+		return g_variant_new_string (cd_object_scope_to_string (priv->object_scope));
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_OWNER) == 0)
+		return g_variant_new_uint32 (priv->owner);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SEAT) == 0)
+		return cd_device_get_nullable_for_string (priv->seat);
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_EMBEDDED) == 0)
+		return g_variant_new_boolean (priv->embedded);
 	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_PROFILING_INHIBITORS) == 0) {
 		bus_names = cd_inhibit_get_bus_names (priv->inhibit);
-		retval = g_variant_new_strv ((const gchar * const *) bus_names, -1);
-		goto out;
+		return g_variant_new_strv ((const gchar * const *) bus_names, -1);
 	}
 
 	/* return an error */
@@ -1675,9 +1570,7 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 		     CD_DEVICE_ERROR_INTERNAL,
 		     "failed to get device property %s",
 		     property_name);
-out:
-	g_strfreev (bus_names);
-	return retval;
+	return NULL;
 }
 
 /**
@@ -1689,8 +1582,7 @@ cd_device_register_object (CdDevice *device,
 			   GDBusInterfaceInfo *info,
 			   GError **error)
 {
-	GError *error_local = NULL;
-	gboolean ret = FALSE;
+	_cleanup_error_free_ GError *error_local = NULL;
 
 	static const GDBusInterfaceVTable interface_vtable = {
 		cd_device_dbus_method_call,
@@ -1713,17 +1605,12 @@ cd_device_register_object (CdDevice *device,
 			     CD_DEVICE_ERROR_INTERNAL,
 			     "failed to register object: %s",
 			     error_local->message);
-		g_error_free (error_local);
-		goto out;
+		return FALSE;
 	}
 	g_debug ("CdDevice: Register interface %i on %s",
 		 device->priv->registration_id,
 		 device->priv->object_path);
-
-	/* success */
-	ret = TRUE;
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -1924,4 +1811,3 @@ cd_device_new (void)
 	device = g_object_new (CD_TYPE_DEVICE, NULL);
 	return CD_DEVICE (device);
 }
-
